import argparse
import notdb
import pyonr
import os
from getpass import getpass
from bcrypt import checkpw as _checkpw
from app import create_app

def read(rel_path):
    import codecs
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()

def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")

v = get_version('__init__.py')

def main():
   pass

def refresh_data(file:pyonr.Read):
   return file.readfile

def get_password():   
   return getpass('Database password: ').encode('utf-8')

def get_port():
   inp = input('Server PORT (default=5000): ')
   if inp:
      return int(inp)
   return 5000

def is_taken_port(port: int):
   # got this from https://stackoverflow.com/questions/2470971/fast-way-to-test-if-a-port-is-in-use-using-python
   import socket
   with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
      return s.connect_ex(('localhost', port)) == 0

parser = argparse.ArgumentParser('NotDB Viewer', 'notdb_viewer [filename]', f'NotDB Viewer tool v{v}')
parser.add_argument('filename', nargs=1, type=str, help='Run a webapp to view your database data', metavar='filename')
parser.add_argument('-v', '--version', action='version', version=f'notdb_viewer {v}', help='Show the notdb_viewer version')

args = parser.parse_args()

if __name__ == '__main__' or __name__ == 'notdb_viewer.__main__':
   if len(args.filename) != 0:
      filename = args.filename[0]
      
      if not os.path.isfile(filename):
         parser.error('Invalid filename/path')
      

      file = pyonr.Read(filename)
      filedata = refresh_data(file)
      db = None
      if filedata.get('__password'):
         _p = get_password()
         if not _checkpw(_p, filedata.get('__password')):
            parser.error('Wrong password.')

         db = notdb.NotDBClient(filename, password=_p)
      else:
         db = notdb.NotDBClient(filename)
            
      
      port = get_port()
      if is_taken_port(port):
         parser.error('Used port')

      wa = create_app()

      wa.db = db
      wa.file = file

      wa.run(port=port)


'''
NotDB Viewer
------------

Simple viewer for NotDB Databases

   $ pip install notdb-viewer

Full documentation is avaliable on [Github](https://github.com/nawafalqari/NotDB_Viewer#readme)
'''

from app import create_app, viewer_html

__version__ = '1.3.0'
__all__ = ['create_app', 'viewer_html']


import notdb

db = notdb.NotDBClient(password='123')

# db.files.appendFile('testImage.png', name='logo')
# db.files.removeFile({'name':'license'})
db.files.appendFile('test.txt', name='t')
# db.files.appendFile('testVideo.mp4', name='video')
# db.files.appendFile('testAudio1.m3u', name='audio1')
# db.files.appendFile('testAudio2.mp3', name='audio2')


'''
NotDB Viewer
------------

Simple viewer for NotDB Databases

    $ pip install notdb-viewer

Full documentation is avaliable on `Github <https://github.com/nawafalqari/NotDB_Viewer#readme>`_.
'''

from setuptools import setup, find_packages
import os.path

# got these 2 functions from https://packaging.python.org/en/latest/guides/single-sourcing-package-version/
def read(rel_path):
    import codecs
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()

def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")

packages = [
    *find_packages()
]

setup(
    name='notdb_viewer',
    packages=packages,
    include_package_data=True,
    install_requires=[
        'notdb>=1.2.6',
        'pyonr>=1.0.0',
        'bcrypt>=3.2.0',
        'flask',
        'flask-minify'
    ],
    version=get_version('notdb_viewer/__init__.py'),
    description='Viewer for NotDB Databases',
    author='Nawaf Alqari',
    author_email='nawafalqari13@gmail.com',
    keywords=['notdb', 'db', 'database', 'notdatabsae', 'simple database'],
    long_description=__doc__,
    long_description_content_type='text/x-rst',
    entry_points={
        'console_scripts': ['notdb_viewer=notdb_viewer.__main__:main']
    },
    license='MIT',
    zip_safe=False,
    url='https://github.com/nawafalqari/NotDB_Viewer/',
    project_urls={
        'Documentation': 'https://github.com/nawafalqari/NotDB_Viewer#readme',
        'Bug Tracker': 'https://github.com/nawafalqari/NotDB_Viewer/issues',
        'Source Code': 'https://github.com/nawafalqari/NotDB_Viewer/',
        'Discord': 'https://discord.gg/cpvynqk4XT',
        'Donate': 'https://paypal.me/NawafHAlqari'
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Topic :: Database'
    ],
)


from flask import Flask, render_template, request, jsonify, Response, make_response
import pyonr
import notdb
from flask_minify import minify

def refresh_data(file:pyonr.Read) -> dict:
   return file.readfile

def get_class(obj):
   return type(obj)

def get_obj_class_name(obj):
   return type(obj)().__class__.__name__

def create_app():

   app = Flask(__name__)
   app.url_map.strict_slashes = False

   minify(app, html=True, js=True, cssless=True, fail_safe=True)

   @app.route('/')
   def documents_route():
      db = app.db
      file = app.file

      data = refresh_data(file)
      db_info = {}

      db_info['Secured with password'] = True if data.get('__password') else False
      db_info['documents'] = db.documents

      return render_template('viewer.html',
                              documents=db.get({}),
                              db_info=db_info,
                              host=db.host,
                              get_obj_class_name=get_obj_class_name,
                              get_class=get_class)

   @app.route('/files')
   def files_route():
      db = app.db
      webAppURL = str(request.url_root)

      return render_template('files.html',
                        files=db.files.getFiles({}),
                        host=db.host,
                        webAppURL=webAppURL,
                        get_obj_class_name=get_obj_class_name,
                        get_class=get_class)

   @app.route('/cdn/<name>')
   def cdn_loader(name):
      try:
         db = app.db

         file = db.files.getFile({'name': name})
         file = db.files.getFile({'name': name})
         err = {}

         if not file:
            err['success'] = False
            err['err'] = 'INVALID_NAME'
            err['msg'] = f'{name} is not in the database'
            return jsonify(err)

         return Response(file['data'], mimetype=file['mimetype'])
      except Exception as e:
         err['err'] = str(e)
         return jsonify(err)


   @app.route('/cdn')
   def cdn_route():
      err = {}
      err['success'] = False
      err['err'] = 'INVALID_NAME'
      err['msg'] = 'add a name after (/cdn/<name>)'

      return jsonify(err)

   return app